#ifndef __AIO_H
#define __AIO_H

#undef FUNC
#ifndef SWIGGY
#define FUNC(x) (*x) 
#else
#define FUNC(x) x
#endif

typedef enum {
  AIO_ADC=1, AIO_DAC=2
} AIOType;

typedef struct AIO AIO;
typedef struct AIOAPI AIOAPI;
typedef struct VoltageRange VoltageRange;
typedef struct VoltageSubRange VoltageSubRange;
typedef struct PeriodRange PeriodRange;

struct VoltageSubRange {
  float start, step;
  unsigned count;
};
struct VoltageRange {
  VoltageSubRange low,high;
};
struct ActualVoltageRange {
  float low, high;
};

struct PeriodRange {
  unsigned start, step, count;
};
#define AIO_PR(start,step,count) ((PeriodRange){start,step,count});
#define AIO_VR(lowstart,lowstep,lowcount,highstart,highstep,highcount) ((VoltageRange){(VoltageSubRange){lowstart,lowstep,lowcount},(VoltageSubRange){highstart,highstep,highcount}})
struct AIOAPI {
  void *FUNC(Init)(void *me);
  void FUNC(Fini)(void *me);
  int FUNC(Lock)(void *me,unsigned num,int flags);
  int FUNC(Unlock)(void *me,unsigned num,int flags);
  int FUNC(Preempt)(void *me);
  AIOType FUNC(Type)(void *me);
  VoltageRange* FUNC(VoltageRangeList)(void *me);
  int FUNC(PrecisionList)(void *me);
  PeriodRange* FUNC(PeriodRangeList)(void *me);
  int* FUNC(TriggerList)(void *me);
  int FUNC(Channels)(void *me);
  int FUNC(Configuration)(void *me,float* low,float* high,int* prec,int* t,int *trigger,int itrig[1]);
  int FUNC(ConfigureTest)(void *me,const float* low,const float* high,const int* prec,const int* rate,const int* trigger,int itrig);
  int FUNC(Configure)(void *me,const float* low,const float* high,const int* prec,const int* rate,const int* trigger,int itrig);
  int FUNC(ITrig)(void *me,int itrig);
  int FUNC(Get)(void *me,int channel);
  int FUNC(Put)(void *me,int channel,int value);
  int FUNC(Ready)(void *me,int channel);
  int FUNC(Gets8)(void *me,char* buf);
  int FUNC(Gets16)(void *me,short* buf);
  int FUNC(Gets32)(void *me,int* buf);
  int FUNC(Puts8)(void *me,const char* buf);
  int FUNC(Puts16)(void *me,const short* buf);
  int FUNC(Puts32)(void *me,const int* buf);
  int FUNC(Readys8)(void *me,const char* buf);
  int FUNC(Readys16)(void *me,const short* buf);
  int FUNC(Readys32)(void *me,const int* buf);
};

struct AIO {
  AIOAPI;
  int InitStatus;
};

#define AIO_HZ(hz) ((int)(1000000000/hz))
enum {
  NetAIO_Lock=0,
  NetAIO_Unlock=1,
  NetAIO_Preempt=2,
  NetAIO_Type=3,
  NetAIO_VoltageRangeList=4,
  NetAIO_PrecisionList=5,
  NetAIO_PeriodRangeList=6,
  NetAIO_TriggerList=7,
  NetAIO_Channels=8,
  NetAIO_Configuration=9,
  NetAIO_ConfigureTest=10,
  NetAIO_Configure=11,
  NetAIO_ITrig=12,
  NetAIO_Get=13,
  NetAIO_Put=14,
  NetAIO_Ready=15,
  NetAIO_Gets8=16,
  NetAIO_Gets16=17,
  NetAIO_Gets32=18,
  NetAIO_Puts8=19,
  NetAIO_Puts16=20,
  NetAIO_Puts32=21,
  NetAIO_Readys8=22,
  NetAIO_Readys16=23,
  NetAIO_Readys32=24
};
#define AIO_APICount (1+NetAIO_Readys32)

#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.

